% This example illustrates the functionality of H-Vis-3D on a minimal
% example of a wild-type (wt) and mutant (mt) simulation of TCRpMHC.
%
% B. Knapp 18.3.2016
%
% 1) Perform your simulations (the output format needs to be xtc, trr, or
% multiple pdb). In our case the simulations are provided in the ./data/
% directory as:
%
% mt1.pdb
% mt1_short.xtc
% mt1.tpr
% mt2.pdb
% mt2_short.xtc
% mt2.tpr
% wt1.pdb
% wt1_short.xtc
% wt1.tpr
% wt2.pdb
% wt2_short.xtc
% wt2.tpr
% where the pdb files are used for visualisation, the xtc files contain
% (short) trajectories, and the tpr files the starting structure of the
% simulations.
%
%
% 2) Next calculate all pairwise H-bonds using Gromacs and the following
% commands with input groups for the interaction partners 1 and 1 ("Protein")
% (also these files are provided in the ./data/ directory):
%
% g_hbond -s wt1.tpr -f wt1_short.xtc -hbn wt1.hbn.ndx -hbm wt1.hbm.xpm -num wt1.num.xvg
% g_hbond -s wt2.tpr -f wt2_short.xtc -hbn wt2.hbn.ndx -hbm wt2.hbm.xpm -num wt2.num.xvg
% g_hbond -s mt1.tpr -f mt1_short.xtc -hbn mt1.hbn.ndx -hbm mt1.hbm.xpm -num mt1.num.xvg
% g_hbond -s mt2.tpr -f mt2_short.xtc -hbn mt2.hbn.ndx -hbm mt2.hbm.xpm -num mt2.num.xvg
% rm -f \#*\# % this cleans the Gromacs mess up
%
% 3) Run the following commands in Matlab:


myDataPath = './data/';
allFullMtxWT = []; % will hold all replicas
allFullMtxMT = [];

for repIter = 1:2 % in our case we have only 2 replicas
    
    fprintf ('working on replica %1.0f\n', repIter);
    
    % wild-type
    baseFn = [myDataPath, 'wt', num2str(repIter)];
    [hBondMatrix, hBondNdxData, hBondNum, pdbWT, allAtomMtxGroup1, allAtomMtxGroup2, allResMtxGroup1, allResMtxGroup2] = ...
        parseHBond_allOutput ...
        ([baseFn, '.hbm.xpm'], [baseFn, '.hbn.ndx'], [baseFn, '.num.xvg'], [baseFn, '.pdb'],  false);
    [fullMtxWT] = createFullHBondMatrix_from_parseHBond_allOutput (hBondNdxData, hBondMatrix, pdbWT);
    allFullMtxWT(:, :, repIter) = fullMtxWT; 

    % mutant
    baseFn = [myDataPath, 'mt', num2str(repIter)];
    [hBondMatrix, hBondNdxData, hBondNum, pdbMT, allAtomMtxGroup1, allAtomMtxGroup2, allResMtxGroup1, allResMtxGroup2] = ...
        parseHBond_allOutput ...
        ([baseFn, '.hbm.xpm'], [baseFn, '.hbn.ndx'], [baseFn, '.num.xvg'], [baseFn, '.pdb'],  false);
    [fullMtxMT] = createFullHBondMatrix_from_parseHBond_allOutput (hBondNdxData, hBondMatrix, pdbMT);
    allFullMtxMT(:, :, repIter) = fullMtxMT;
    
end

allFullMtxMeanWT = mean (allFullMtxWT, 3); % average over replicas (one could also use the median, modus, trend, max, min etc here)
allFullMtxMeanMT = mean (allFullMtxMT, 3);


additionalVmdStr = '\n mol addrep 0 \n mol modselect 5 0 chain A and resid 65 and type CA \n mol modstyle 5 0 VDW 1.000000 12.000000 \n mol modcolor 5 0 ColorID 4 \n mol modstyle 5 0 VDW 0.700000 12.000000 \n';

fprintf ('\nPlease note that the following visualisations do not reproduce the figures of the manuscript as the here produced visualisations are only based on a minimal dataset of 2 replicas and 100 frames of each simulation to illustrate the principle. The full dataset is 771 GB in size and can be obtained from the authors upon request.\n\n');

writeVmdFileForHBondPlots_basedOnMtx (allFullMtxMeanWT, pdbWT, 'vmd_visualisation_WT_H_bond.txt', 0.0001, additionalVmdStr); 
writeVmdFileForHBondPlots_basedOnMtx (allFullMtxMeanMT, pdbMT, 'vmd_visualisation_MT_H_bond.txt', 0.0001, additionalVmdStr);

diffMtx = allFullMtxMeanWT - allFullMtxMeanMT; % calculate the difference between wild-type and mutant
writeVmdFileForHBondPlots_basedOnMtx (diffMtx, pdbWT, 'vmd_visualisation_WTvsMT_H_bond.txt', 0.0001, additionalVmdStr);

% 4) Visualise the results in VMD:
% Visualisation written to "vmd_visualisation_WT_H_bond.txt". The content of this file can be copy&pasted into VMD => extensions => Tk Console. 
% Visualisation written to "vmd_visualisation_MT_H_bond.txt". The content of this file can be copy&pasted into VMD => extensions => Tk Console. 
% Visualisation written to "vmd_visualisation_WTvsMT_H_bond.txt". The content of this file can be copy&pasted into VMD => extensions => Tk Console.
