function [hBondNdxData] = parseHBond_hbn (fileName)
% function parses a file created by the g_hbond function of gromacs via the
% -hbn option.
%
% How to parse the h-bonds: The -num option can be parsed with the function
% "parseXvgFile.m" and the -hbm with the function "parseHBond_hbm.m".
% The -hbn option with the function "parseHBond_hbn.". Further information
% on the atom type and coordinates etc can be obtained via "parsePdb.m"
% 
% example for input file (identical groups, in both cases "protein"):
% [ Protein ]
%     1     2     3     4     5     6     7     8     9    10    11    12    13    14    15
%    16    17    18    19    20    21    22    23    24    25    26    27    28    29    30
%    31    32    33    34    35    36    37    38    39    40    41    42    43    44    45
%    46    47    48    49    50    51    52    53    54    55    56    57    58    59    60
%    61    62    63    64    65    66    67    68    69    70    71    72    73    74    75
%    ...
%  6406  6407  6408  6409  6410  6411  6412  6413  6414  6415  6416  6417  6418  6419  6420
%  6421  6422  6423  6424  6425
% [ donors_hydrogens_Protein ]
%     3    4    3    5    3    6
%    10   11
%    18   19
%    28   29
% ....
% [ acceptors_Protein ]
%     3     9    10    14    17    18    23    28    31    32    36    39    40    48    49
%    55    58    61    65    66    80    83    84   100   101   117   118   122   126   127
%   131   134   135   142   143   147   150   151   157   160   163   167   168   174   175
% ...
% [ hbonds_Protein ]
%       3      4     23
%       3      4   1119
%       3      4   1122
% ...
%
%  examples for h-bonds between chain D+E and chain C:
% [ hbonds_chD_chE-chC ]
%    4636   4637   4133
%    7177   7178   4118
%    7172   7174   4118
%    4443   4444   4133
%    7197   7198   4141
%    4137   4138   7164
%    4091   4092   5083
%    4083   4085   5078
%    4133   4134   7182
%
% The first column contains usually the donor heavy atom, the second one the donor
% H atom and the third one the acceptor. (if you cross check this with vmd
% be carefull with the zero vs one index!)
% 
% the following output struct will be created:
% 
% hBondMatrix = 
% 
%        group1Name: 'chD_chE' % name of the first group as given in the index file
%        group2Name: 'chC' % name of the second group as given in the index file
%     group1AtomNrs: [1x4545 double] % all atom numbers of atoms being part of group 1. In this case a TCR
%     group2AtomNrs: [1x97 double] % all atom numbers of atoms being part of group 2. In this case the peptide
%            donors: [9x2 double] % the atom numbers (of group 1 and/or 2) which are donors (always the H and the heavy atom)
%         acceptors: [9x1 double] % the atom numbers (of group 1 and/or 2) which are acceptors (the heavy atom)
% 
% example:
%
% myPath = '/run/media/knapp/Seagate Expansion Drive/1mi5_172_100ns_sims/1A/';
% hBondNdxData = parseHBond_hbn ([myPath, '1A.hbondNdx.TCRvsPeptide.ndx']);
% 
% hBondNdxData = parseHBond_hbn('/data/housemartin/knapp/matlabWorkspace/util/parsing/ascii/g5_h-bond/test.hbn.ndx');
%
% B. Knapp 2013-08-20

[data, ~] = readFile (fileName);

lineIter = 1;

targetHBonds = '[ hbonds';
hBondSectionFound = false;

donors = [];
acceptors = [];

while lineIter <= length(data)
    line = data{lineIter};
    if length(line) > 7
        
        if hBondSectionFound % the hbond section was already found in a previous iteration. now we can extract the values
            vals = textscan(line, '%f');
            vals = vals {1};
            donors (end+1, :) = vals (1:2);
            acceptors (end+1) = vals (3);
        end
        
        if strcmp (line(1:8), targetHBonds) % we found the target hbond section and extract the header information first
            % e.g. [ hbonds_chD_chE-chC ]
            hBondSectionFound = true;
            group1StartIdx = length(targetHBonds) + 2;
            group1EndIdx = strfind(line, '-') - 1;
            % this can lead to multiple indices if e.g. [ hbonds_chA_&_r_1-180_chC_chD_chE-chA_&_r_1-181_chC_chD_chE ]
            % the "-" between 2 numerics is not what we want. therefore we delete multiple "-" indices if the "-" is after a numeric e.g. "1-"
            delIdx = [];
            for group1EndIdxIter = 1:size(group1EndIdx,2)
                prevPos = line(group1EndIdx(group1EndIdxIter));
                num = str2double(prevPos);
                if ~isnan(num) % if it is not not a number (is a number) then delete our index
                    delIdx(end+1) = group1EndIdxIter; 
                end
            end
            group1EndIdx(delIdx) = [];
            
            if ~isempty (group1EndIdx)
                % case: we have 2 different group e.g. [ hbonds_chD_chE-chC ]
                group2StartIdx = group1EndIdx + 2;
                group2EndIdx = strfind(line, ']') - 2;
            else
                 % case: we have only 1 group e.g. [ hbonds_chA ] (there is no "-", that means we are doing h-bonds with a self group)
                 % also works for [ hbonds_chA_&_r_1-180_chC_chD_chE-chA_&_r_1-181_chC_chD_chE ] (there is no valid "-", that means we are doing h-bonds with a self group)
               group1EndIdx = length(line)-2;
               group2StartIdx = group1StartIdx;
               group2EndIdx = group1EndIdx;
            end

            group1Name = line(group1StartIdx:group1EndIdx);
            group2Name = line(group2StartIdx:group2EndIdx);
            hBondNdxData.group1Name = group1Name;
            hBondNdxData.group2Name = group2Name;
        end
    end
    lineIter = lineIter + 1;
end

lineIter = 1; % now we know the chains (group1Name, group2Name) => check which atoms are in there
target1 = ['[ ', group1Name ,' ]'];
target2 = ['[ ', group2Name ,' ]'];

group1AtomNrs = [];
group2AtomNrs = [];

while lineIter <= length(data)
    line = data{lineIter};
    if length(line) > 6
        if strcmp(line, target1) % parse the first group
            lineIter = lineIter + 1;
            line = data{lineIter};
            while not(strcmp(line(1:2), '[ '))
                vals = textscan(line, '%f');
                if lineIter == 201
                    
                   a = 1;
                end
                group1AtomNrs = [group1AtomNrs, vals{1}'];
                lineIter = lineIter + 1;
                line = data{lineIter};
            end % while
        end % if
        
        if strcmp(line, target2) % parse the first group
            lineIter = lineIter + 1;
            line = data{lineIter};
            while not(strcmp(line(1:2), '[ '))
                vals = textscan(line, '%f');
                group2AtomNrs = [group2AtomNrs, vals{1}'];
                lineIter = lineIter + 1;
                line = data{lineIter};
            end % while
        end % if
    
    end
    lineIter = lineIter + 1;
end

if isempty(group2AtomNrs)
    group2AtomNrs = group1AtomNrs; % the groups are identical therefore we copy
end

hBondNdxData.group1AtomNrs = group1AtomNrs;
hBondNdxData.group2AtomNrs = group2AtomNrs;

% if strcmp(hBondNdxData.group1Name, hBondNdxData.group2Name)
%     donors = donors / 2; % due to a symetric matrix the values are exatly the double
%     acceptors = acceptors / 2;
% end

hBondNdxData.donors = donors;
hBondNdxData.acceptors = acceptors';


